@@ -2,12 +2,15 @@ from __future__ import print_function
 import random
 import time
 import sys
+import os
 
 from metaflow.exception import MetaflowException
 from metaflow.metaflow_config import S3_ENDPOINT_URL, S3_VERIFY_CERTIFICATE
 
 S3_NUM_RETRIES = 7
 
+TEST_S3_RETRY = 'TEST_S3_RETRY' in os.environ
+
 def get_s3_client():
     from metaflow.plugins.aws.aws_client import get_aws_client
     return get_aws_client(
@@ -21,7 +24,13 @@ def aws_retry(f):
         last_exc = None
         for i in range(S3_NUM_RETRIES):
             try:
-                return f(self, *args, **kwargs)
+                ret = f(self, *args, **kwargs)
+                if TEST_S3_RETRY and i == 0:
+                    raise Exception("TEST_S3_RETRY env var set. "
+                                    "Pretending that an S3 op failed. "
+                                    "This is not a real failure.")
+                else:
+                    return ret
             except MetaflowException as ex:
                 # MetaflowExceptions are not related to AWS, don't retry
                 raise
@@ -35,7 +44,8 @@ def aws_retry(f):
                                  % (function_name, ex, S3_NUM_RETRIES - i))
                 self.reset_client(hard_reset=True)
                 last_exc = ex
-                # exponential backoff
-                time.sleep(2**i + random.randint(0, 5))
+                # exponential backoff for real failures
+                if not (TEST_S3_RETRY and i == 0):
+                    time.sleep(2**i + random.randint(0, 5))
         raise last_exc
-    return retry_wrapper
+    return retry_wrapper
\ No newline at end of file
