@@ -15,7 +15,7 @@ except NameError:
     basestring = str
 
 from metaflow.exception import MetaflowException
-from metaflow.metaflow_config import AWS_SANDBOX_ENABLED
+from metaflow.metaflow_config import AWS_SANDBOX_ENABLED, METADATA_SERVICE_AUTH_KEY
 
 class BatchClient(object):
     def __init__(self):
@@ -92,6 +92,16 @@ class BatchJob(object):
                                               self._shared_memory,
                                               self._max_swap,
                                               self._swappiness)
+
+        try:
+            aero_user = METADATA_SERVICE_AUTH_KEY.split(":")[0]
+        except:
+            aero_user = "default"
+
+        self.payload['tags'] = {
+            "UserId": aero_user
+        }
+        
         response = self._client.submit_job(**self.payload)
         job = RunningJob(response['jobId'], self._client)
         return job.update()
@@ -188,8 +198,13 @@ class BatchJob(object):
         #             job_definition['containerProperties'] \
         #                 ['linuxParameters']['maxSwap'] = int(max_swap)
 
+        try:
+            aero_user = METADATA_SERVICE_AUTH_KEY.split(":")[0]
+        except:
+            aero_user = "default"
+
         # check if job definition already exists
-        def_name = 'metaflow_%s' % \
+        def_name = aero_user + '_metaflow_%s' % \
             hashlib.sha224(str(job_definition).encode('utf-8')).hexdigest()
         payload = {'jobDefinitionName': def_name, 'status': 'ACTIVE'}
         response = self._client.describe_job_definitions(**payload)
@@ -477,6 +492,9 @@ class RunningJob(object):
     def logs(self):
         def get_log_stream(job):
             log_stream_name = job.log_stream_name
+
+            print(log_stream_name)
+
             if log_stream_name:
                 return BatchLogs('/aws/batch/job', log_stream_name, sleep_on_no_data=1)
             else:
