@@ -4,6 +4,7 @@ from hashlib import sha1
 import json
 import re
 from distutils.version import LooseVersion
+from datetime import datetime
 
 from metaflow import current, decorators, parameters, JSONType
 from metaflow.metaflow_config import SFN_STATE_MACHINE_PREFIX
@@ -17,6 +18,8 @@ from .step_functions import StepFunctions
 from .production_token import load_token, store_token, new_token
 
 VALID_NAME = re.compile('[^a-zA-Z0-9_\-\.]')
+BACK_KEYWORD = "Back"
+EXIT_KEYWORD = "Exit"
 
 class IncorrectProductionToken(MetaflowException):
     headline = "Incorrect production token"
@@ -424,3 +427,321 @@ def list_runs(obj,
         else:
             obj.echo('No executions for *%s* found on AWS Step Functions.' \
                 % (obj.state_machine_name))
+
+
+@schedule.command(
+    help="List all runs of the workflow on Aero scheduler.")
+@click.option("--flow-id", default=None,
+    help="List all runs of the workflow in ABORTED state on "
+        "the Aero scheduler.")
+@click.pass_obj
+def get_logs(obj, flow_id=None):
+
+    from ..aws_client import get_aws_client 
+
+    # Get AWS Clients
+    sfn = get_aws_client('stepfunctions')
+    cloudwatch = get_aws_client('logs')
+
+    cli_state = {
+        "flow_id": obj.flow.name
+    }
+
+    # Execution (Run) > Step > Task
+    question_arr = [
+        ask_execution_question, 
+        ask_step_question, 
+        ask_task_question
+    ]
+
+    # Iterate through question array, reducing question if BACK selected
+    # If question returns false, reduce question number to prevent loop
+    question = 0
+    while question >= 0:
+
+        resp = question_arr[question](
+            cli_state,
+            sfn=sfn,
+            cloudwatch=cloudwatch
+        )
+
+        # If back or 
+        if resp == BACK_KEYWORD:
+            question -= 1
+        # If we are at the end, stay on the last question
+        elif question != (len(question_arr)-1):
+            question += 1
+
+
+def get_key(param, key):
+    try:
+        lst = param['Container']['Environment']
+
+        for elem in lst:
+            if elem['Name'] == key:
+                return elem['Value']
+
+        return 0
+    except:
+        return 0
+
+
+def build_event_tree(execution_history):
+
+    tree = {}
+
+    # Events are in order
+    for task in execution_history['events']:
+
+        # If task has scheduled 
+        if task['type'] == "TaskScheduled":
+        
+            # Some intermediate tasks arent actually steps - so skip
+            if task['taskScheduledEventDetails'].get('resourceType', "") == "dynamodb":
+                continue
+
+            # Get step name
+            param = json.loads(task['taskScheduledEventDetails']['parameters'])
+            step_name = param['Parameters']['metaflow.step_name']
+
+            tree[step_name] = {}
+
+        # If Succeeded
+        if task['type'] == "TaskSucceeded":
+
+            # Some intermediate tasks arent actually steps - so skip
+            if task['taskSucceededEventDetails'].get('resourceType', "") == "dynamodb":
+                continue
+
+            output = json.loads(task['taskSucceededEventDetails']['output'])
+            step_name = output['Parameters']['metaflow.step_name']
+
+            # Could be a FOR step, so get the task ids for each foreach task. If not then 0.
+            split_index = int(get_key(output, 'METAFLOW_SPLIT_INDEX'))
+
+            # Tasks can be attempted multiple times, so iterate through these
+            for attempt in output['Attempts']:
+                
+                # If split index in the step dict already
+                if split_index in tree[step_name]:
+                    # Add task state and logid
+                    tree[step_name][split_index].append(("TaskSucceeded", attempt['Container']['LogStreamName']))
+                else:
+                    # Add task state and logid
+                    tree[step_name][split_index] = [("TaskSucceeded", attempt['Container']['LogStreamName'])]
+                    
+        if task['type'] == "TaskFailed":
+
+            if task['taskFailedEventDetails'].get('resourceType', "") == "dynamodb":
+                continue
+
+            output = json.loads(task['taskFailedEventDetails']['cause'])
+            step_name = output['Parameters']['metaflow.step_name']
+
+            split_index = int(get_key(output, 'METAFLOW_SPLIT_INDEX'))
+
+            # Tasks can be attempted multiple times, so iterate through these
+            for attempt in output['Attempts']:
+
+                # If split index in the step dict already
+                if split_index in tree[step_name]:
+                    tree[step_name][split_index].append(("TaskFailed", attempt['Container']['LogStreamName']))
+                else:
+                    tree[step_name][split_index] = [("TaskFailed", attempt['Container']['LogStreamName'])]
+
+    '''
+        End up with:
+        {
+            "<step_name>": { # Step name
+                0: [ # Attempt id
+                    (TaskState, LogId),
+                    ...
+                ],
+                ..
+            },
+            ...
+        }
+    '''
+                    
+    return tree
+   
+
+def ask_task_question(obj, cloudwatch=None, sfn=None):
+
+    # If there is only one task (likely for most steps), open the task logs
+    if len(obj['tree'][obj['step']].keys()) == 1:
+        
+        # fetch logs for tree[step][0][0][1]
+        open_log(obj['tree'][obj['step']][0][0][1], cloudwatch)
+        # If we dont BACK, then this function keeps being run, then opening logs
+        # So we go back to get to step selector
+        return BACK_KEYWORD
+
+    task = ask_questions(
+        f"Which Task from Step {obj['step']}?",
+        [f"{obj['step']}_{i}" for i in obj['tree'][obj['step']].keys()],
+        enable_back=True
+    )
+
+    if check_if_back_or_exit(task):
+        return BACK_KEYWORD
+
+    task_id = int(task.split("_")[-1])
+
+    open_log(obj['tree'][obj['step']][task_id][0][1], cloudwatch)
+
+
+def ask_step_question(obj, cloudwatch=None, sfn=None):
+
+    execution_history = sfn.get_execution_history(
+        executionArn=obj['execution_arn']
+    )
+
+    tree = build_event_tree(execution_history)
+
+    steps = []
+
+    # Add Failed tag to steps which have failed
+    for key in tree:
+        has_failed = False
+        for index in tree[key]:
+            for attempts in tree[key][index]:
+                if attempts[0] == "TaskFailed":
+                    has_failed = True
+        
+        steps.append(f"FAILED: {key}" if has_failed else key)
+
+    step = ask_questions(
+        f"Which Step in {obj['flow_id']}?",
+        steps,
+        enable_back=True
+    ).replace("FAILED: ", "")
+
+    if check_if_back_or_exit(step):
+        return BACK_KEYWORD
+
+    obj['step'] = step
+    obj['tree'] = tree
+
+
+def ask_execution_question(obj, cloudwatch=None, sfn=None):
+
+    # Get all StateMachines from AWS
+    functions = sfn.list_state_machines()['stateMachines']
+    functions.sort(key=lambda item:item['creationDate'], reverse=True)
+
+    def get_key(lst, key, name):
+
+        for elem in lst:
+            if elem[key] == name:
+                return elem
+
+        return None
+
+    flow_state_machine = get_key(functions, "name", obj['flow_id'])
+
+    executions = sfn.list_executions(
+        stateMachineArn=flow_state_machine['stateMachineArn']
+    )['executions']
+    executions.sort(key=lambda item:item['startDate'], reverse=True)
+
+    execution_date = ask_questions(
+        f"Which execution of {obj['flow_id']}?",
+        [x['startDate'] for x in executions]
+    )
+
+    if check_if_back_or_exit(execution_date):
+        return BACK_KEYWORD
+
+    execution_arn = get_key(executions, 'startDate', execution_date)['executionArn']
+
+    obj['execution_arn'] = execution_arn
+
+
+def check_if_back_or_exit(answer):
+
+    if answer == BACK_KEYWORD:
+        return True
+    if answer == EXIT_KEYWORD:
+        import sys
+        sys.exit(0)
+    
+def chunked(lst, n):
+    return [lst[i:i + n] for i in range(0, len(lst), n)]
+
+def generate_control_opts(i, l, enable_back=False):
+    lst = [BACK_KEYWORD] if enable_back else [] 
+
+    if i != 0:
+        lst.append("Previous Page")
+    if i != (l-1):
+        lst.append("Next Page")
+    
+    lst.append(EXIT_KEYWORD)
+    
+    return lst
+
+def ask_questions(question, choices, enable_back=False):
+    import inquirer
+
+    chunked_choices = chunked(choices, 8)
+    i = 0
+
+    while True:
+        questions = [
+            inquirer.List("var",
+                        message=question,
+                        choices=chunked_choices[i] + \
+                            generate_control_opts(i, len(chunked_choices), enable_back),
+                    ),
+        ]
+        response =  inquirer.prompt(questions)["var"]
+
+        if response == "Next Page":
+            i +=1
+        elif response == "Previous Page":
+            i -=1
+        else:
+            return response
+        
+def open_log(log_stream, cloudwatch):
+
+    resp = cloudwatch.get_log_events(
+        logGroupName="/aws/batch/job",
+        logStreamName=log_stream
+    )
+    token = ""
+
+    logs = resp['events']
+
+    while token != resp['nextForwardToken']:
+        token = resp['nextForwardToken']
+
+        resp = cloudwatch.get_log_events(
+            logGroupName="/aws/batch/job",
+            logStreamName=log_stream,
+            nextToken=token
+        )
+        logs.extend(resp['events'])
+    
+    str_logs = ""
+
+    for log in logs:
+        time = datetime.utcfromtimestamp(log['timestamp']/1000)
+        str_logs += f"{time.isoformat()}: {log['message']}\n"
+
+    import os, subprocess, tempfile
+
+    fd, path = tempfile.mkstemp()
+    try:
+        with os.fdopen(fd, 'w') as tmp:
+            tmp.write(str_logs)
+        
+        cmd = os.environ.get('EDITOR', 'vi') + ' ' + path
+        subprocess.call(cmd, shell=True)
+    finally:
+        os.remove(path)
+
+
+
+
